#include "globals.h"
#include "dlgstatus.h"
#include "../paths.h"
#include <iostream>
#include "customevents.h"
#include <qapplication.h>

#include <qthread.h>

using namespace std;

DlgStatus::DlgStatus(QWidget *parent, const char *name)
		: QWidget(parent, name) {
	localDir = new QDir();
	gridMain = new QGridLayout(this, 4, 3);
	leftList = new QListView(this, "left side file box");
	rightList = new QListView(this, "right side file box");
	transferList = new QListView(this, "transferrable file list");
	cout << "ADDRESS OF QThread::currentThread() in con: " << QThread::currentThread() << endl;
	cout << "ADDRESS OF QThread::currentThread() in con: " << QThread::currentThread() << endl;
	cout << "ADDRESS OF QThread::currentThread() in con: " << QThread::currentThread() << endl;
	fileProgress = new QProgressBar(100, this, "file progress bar");
	operationsPanel = new QVBox(this, "operations box");
	statusBar = new QStatusBar(this, "status bar");
	imgDownload = new QPixmap(QString("%1/share/pixmaps/qsync/arrowLeft.png").arg(PREFIX));
	imgUpload = new QPixmap(QString("%1/share/pixmaps/qsync/arrowRight.png").arg(PREFIX));
	btnDownload = new QPushButton("<==", operationsPanel, "download button");
	btnUpload = new QPushButton("==>", operationsPanel, "upload button");
	nameHeader = "Name";
	permsHeader = "Perms";
	originalDir = localDir->absPath();
	gridMain->addWidget(leftList, 0, 0);
	gridMain->addWidget(operationsPanel, 0, 1);
	gridMain->addWidget(rightList, 0, 2);
	gridMain->addMultiCellWidget(transferList, 1, 1, 0, 2);
	gridMain->addMultiCellWidget(fileProgress, 2, 2, 0, 2);
	gridMain->addMultiCellWidget(statusBar, 3, 3, 0, 2);
	rightList->addColumn(nameHeader);
	rightList->addColumn("Size");
	rightList->addColumn("Time");
	rightList->addColumn("Date");
	rightList->addColumn(permsHeader);
	leftList->addColumn(nameHeader);
	leftList->addColumn("Size");
	leftList->addColumn("Time");
	leftList->addColumn("Date");
	leftList->addColumn(permsHeader);
	transferList->addColumn("Filename");
	transferList->addColumn("Size");
	transferList->addColumn("Percent Completed");
	transferList->addColumn("Time");
	transferList->addColumn("Transfer Rate");

	new QListViewItem(transferList, "blah", "1", "2", "3", "4");

	connect(rightList, SIGNAL(selectionChanged()), this, SLOT(remoteSelectChanged()));
	connect(rightList, SIGNAL(doubleClicked(QListViewItem *)), this, SLOT(remoteListDoubleClicked(QListViewItem *)));
	connect(rightList, SIGNAL(returnPressed(QListViewItem *)), this, SLOT(remoteListDoubleClicked(QListViewItem *)));
	connect(leftList, SIGNAL(doubleClicked(QListViewItem *)), this, SLOT(localListDoubleClicked(QListViewItem *)));
	connect(leftList, SIGNAL(returnPressed(QListViewItem *)), this, SLOT(localListDoubleClicked(QListViewItem *)));
	connect(btnDownload, SIGNAL(clicked()), this, SLOT(downloadClicked()));
	connect(btnUpload, SIGNAL(clicked()), this, SLOT(uploadClicked()));

	btnDownload->setPixmap(*imgDownload);
	btnUpload->setPixmap(*imgUpload);
	btnDownload->setFixedSize(imgDownload->width() + 20, imgDownload->height() + 20);
	btnUpload->setFixedSize(imgUpload->width() + 20, imgUpload->height() + 20);
	fillLocalList();
	remoteSelectChanged();
}

void DlgStatus::customEvent(QCustomEvent *e) {
	cout << "DlgStatus::customEvent HANDLE: " << QThread::currentThread() << endl;
	QStringList *args;
	QString filename;
	switch (e->type()) {
		case EVENT_ADD_REMOTE:
			args = (QStringList *)e->data();
			addListItem((*args)[0], (*args)[1], (*args)[2], (*args)[3]);
			break;
		case EVENT_SET_CURRENT:
			cout << "EVENT_SET_CURRENT" << endl;
			cout << "EVENT_SET_CURRENT" << endl;
			cout << "EVENT_SET_CURRENT" << endl;
			filename = (char *)e->data();
			addTransferItem("blah fsdfsd");
			addTransferItem("blah fsdfsd");
			addTransferItem("blah fsdfsd");
			//addListItem("1", "2", "3", "4");
			//setCurrFile(filename);
			break;
		default:
			cerr << "ERROR: bad customEvent \"" << e->type() << "\" in " << __FILE__ << ":" << __LINE__ << ".  You should never see this!" << endl;
			break;
	}
}

void DlgStatus::fillLocalList() {
	leftList->clear();
	QString perms;
	QFileInfoList *fileList = (QFileInfoList *) localDir->entryInfoList();
	for (QFileInfo *fInfo = fileList->first(); fInfo; fInfo = fileList->next()) {
		if (fInfo->fileName() != ".") {
			// time to list all permission bits
			if (fInfo->isSymLink()) perms = "l";
			else if (fInfo->isDir()) perms = "d";
			else perms = "-";
			if (fInfo->permission(QFileInfo::ReadUser)) perms += "r";
			else perms += "-";
			if (fInfo->permission(QFileInfo::WriteUser)) perms += "w";
			else perms += "-";
			if (fInfo->permission(QFileInfo::ExeUser)) perms += "x";
			else perms += "-";
			if (fInfo->permission(QFileInfo::ReadGroup)) perms += "r";
			else perms += "-";
			if (fInfo->permission(QFileInfo::WriteGroup)) perms += "w";
			else perms += "-";
			if (fInfo->permission(QFileInfo::ExeGroup)) perms += "x";
			else perms += "-";
			if (fInfo->permission(QFileInfo::ReadOther)) perms += "r";
			else perms += "-";
			if (fInfo->permission(QFileInfo::WriteOther)) perms += "w";
			else perms += "-";
			// Can't use QFileInfo::permission(int) for sticky bit.
			if (fInfo->permission(QFileInfo::ExeOther)) perms += "x";
			else perms += "-"; // Finally, done!  Condense this better later.
			new QListViewItem(leftList, fInfo->fileName(), QString("%1").arg(fInfo->size()),
				fInfo->lastModified().time().toString(), fInfo->lastModified().date().toString(), perms);
		}
	}
}

void DlgStatus::addListItem(const char *perms, const char *size, const char *modtime, const char *name) {
	if (strcmp(".", name)) {
		QString timeDate = modtime;
		new QListViewItem(rightList, name, size, timeDate.section(' ', 1, 1), timeDate.section(' ', 0, 0), perms);
	}
	else
		cout << "Not adding \".\"." << endl;
}

void DlgStatus::addTransferItem(QString name) {
	cout << "adding item \"" << name << "\"" << endl;
	new QListViewItem(transferList, "something3", "5", "6", "5", "3");
	//transferList->insertItem(new QListViewItem(transferList, "blah", "2", "3", "4", "5"));
	cout << "ADDRESS OF thread: " << QThread::currentThread() << endl;
	cout << "ADDRESS OF thread: " << QThread::currentThread() << endl;
	cout << "ADDRESS OF thread: " << QThread::currentThread() << endl;
	cout << "added item" << endl;
}

void DlgStatus::localListDoubleClicked(QListViewItem *item) {
	QString temp = localDir->absPath() + "/" + item->text(0);
	if (QFileInfo(temp).isDir() && !QFileInfo(temp).isSymLink()) {
		localDir->cd(temp);
		leftList->clear();
		fillLocalList();
	}
	else
		emit syncSendFile(temp.remove(0, temp.findRev('/') + 1)); // get item after last slash
}

void DlgStatus::remoteListDoubleClicked(QListViewItem *item) {

	// perms bitfield lists as (sticky) directory
	if (item->text(4)[0] == 'd' || item->text(4)[0] == 't')
		emit syncViewDir(item->text(0));
	else
		emit syncGetFile(item->text(0));
}

void DlgStatus::changeStatus(QString s) {
	statusBar->clear();
	statusBar->message(s);
}

void DlgStatus::clearRecvList() {
	rightList->clear();
	new QListViewItem(rightList, "..", "", "", "", "d---------");
	new QListViewItem(rightList, "bootdisks", "", "", "", "d---------");
	remoteSelectChanged();
}

void DlgStatus::clearTransList() {
	transferList->clear();
}

int DlgStatus::setCurrFile(QString file) {
	bool matched = false;
	if (((currTransfer = transferList->firstChild()) == transferList->lastItem())
			&& currTransfer->text(0) == file)
		matched = true;
	else {
		for (currTransfer = transferList->firstChild(); currTransfer; currTransfer = currTransfer->nextSibling()) {
			if (currTransfer->text(0) == file) {
				matched = true; // found a match, all good
				break;
			}
		}
	}
	transferList->ensureItemVisible(currTransfer);
	if (matched)
		return 0;
	else
		return 1; // failures shouldn't happen, but...
}

void DlgStatus::refreshLocal() {
	fillLocalList();
}

void DlgStatus::setCWD(QString dir) {
	if ("" != dir) {
		localDir->setPath(dir);
	}
	else {
		localDir->setPath(originalDir);
	}
}

QString DlgStatus::getCWD() {
	return localDir->absPath();
}

void DlgStatus::setFileProgress(long size, int percent, QString rate, QString eta) {
	fileProgress->setProgress(percent);
	currTransfer->setText(1, QString("%1").arg(size));
	currTransfer->setText(2, QString("%1").arg(percent));
	currTransfer->setText(3, eta);
	currTransfer->setText(4, rate);
}

void DlgStatus::makeFileComplete() {
	currTransfer->setText(2, "100");
}

void DlgStatus::downloadClicked() {
	emit syncGetFile(rightList->selectedItem()->text(0));
}

void DlgStatus::uploadClicked() {
	emit syncSendFile(leftList->selectedItem()->text(0));
}

void DlgStatus::remoteSelectChanged() {
	if (rightList->selectedItem())
		btnDownload->setEnabled(true);
	else
		btnDownload->setEnabled(false);
}
