#ifndef SYNCPROCESS_H
#define SYNCPROCESS_H

#include "globals.h"
#include "customevents.h"
#include "siteinfo.h"
#include "dlgstatus.h"
#include "dlgmodulelist.h"
#include "syncthread.h"
#include <qdatetime.h>
#include <qevent.h>
#include <qfile.h>
#include <qpushbutton.h>
#include <qvbox.h>
#include <qvaluelist.h>

class SyncProcess : public QObject {
		Q_OBJECT
	public:
		SyncProcess(const SiteInfo &si, DlgStatus &ds, DlgModuleList &dml);
		void go(RSyncStuff::RSyncOperation op);
	public slots:
		void addRsyncSpew(QString s);
		void viewDir(QString s);
		void getFile(QString s);
		void sendFile(QString s);
		void setNoModuleSpecified(bool b);
	protected:
		void customEvent(QCustomEvent *e);
	signals:
		void percentChanged(int);
		void moduleListShown();
		void transferDialogShown();
	private slots:
		void postSync();
	private:
		void printStatus(QString s, bool isError=false, bool addEndl=true);
		void processMOTDSpew();
		void displayMOTD();
		// TODO: unnecessary other than testing? void displayShares();
		enum ArgType { Command, Option, Source, Destination, Remote, Local };
		void resetState();
		void runProcess();
		void syncOperation(RSyncStuff::RSyncOperation op, QString s);
		SiteInfo *siteInfo;
		DlgStatus *statusDialog;
		DlgModuleList *moduleListDialog;
		SyncThread *rsyncThread;
		bool readingOutput, readingError, readyOutput, readyError,
			connectDone,
			genRunning, genFilesP1, genFilesP2, recvDone, fileMatched,
			ranOnce, modeIsDownload, noModuleSpecified,
			syncRunning;
		// new vars... above ones may go away
		bool recvFileInfo, motdNext, motdReceived, recvListStart, recvListDone,
			xferringFiles, listingOnly, listingMods;
		// new bools for uploading parsing
		bool uploading;
		int fileCountList, fileRemovedList, filesRemain, generatorPID, motdCounter;
		QDateTime syncStartTime, syncEndTime, fileStartTime, fileEndTime;
		QString currentFile;
		QStringList filenames, motd, output, argList, motdModuleSpew, moduleList;
		QStringList::iterator itFilenames;
		QValueList<ArgType> argTypeList; 
};

#endif /* SYNCPROCESS_H */
